import 'package:copy_with_extension/copy_with_extension.dart';
import 'package:flutter/material.dart';
import 'package:healthrian_visualization_support/healthrian_visualization_support.dart';

part 'ble_scanner.g.dart';

_$BleScannerCWProxy get ScannerWearEcg =>
    BleScanner(module: BLEManager().wearEcg12.module).copyWith;

_$BleScannerCWProxy get ScannerCardioClick =>
    BleScanner(module: BLEManager().cardioClick.module).copyWith;

_$BleScannerCWProxy get ScannerWearT =>
    BleScanner(module: BLEManager().wearCardioT.module).copyWith;

@CopyWith()
class BleScanner extends StatefulWidget {
  const BleScanner({
    super.key,
    this.module,
    this.connectedBuilder,
    this.scannedBuilder,
    this.onTapConnected,
    this.onTapScanned,
    this.emptyWidget,
    this.scanFrequency = 3,
    this.enableTapConnected = true,
    this.enableTapScanned = true,
  });

  final ModuleInterface? module;

  final Function(BluetoothDevice)? onTapConnected;
  final Function(ScanResult)? onTapScanned;
  final Widget Function(BluetoothDevice)? connectedBuilder;
  final Widget Function(ScanResult)? scannedBuilder;

  final Widget? emptyWidget;
  final int scanFrequency;

  final bool enableTapConnected;
  final bool enableTapScanned;

  @override
  State<BleScanner> createState() => _BleScannerState();
}

class _BleScannerState extends State<BleScanner> {
  @override
  Widget build(BuildContext context) {
    final module = widget.module ?? BLEManager().scanner;
    return Scaffold(
      backgroundColor: Colors.transparent,
      body: StreamBuilder(
        stream: () async* {
          final stream =
              Stream.periodic(Duration(seconds: widget.scanFrequency));
          await for (final _ in stream) {
            final connected = await module.ble.findConnectedDevices();
            final scanned = await module.ble.findScannedDevices();

            final isUnBondingRequired = (scanResult) => connected
                .map((device) => device.remoteId)
                .contains(scanResult.device.remoteId);
            scanned.removeWhere(isUnBondingRequired);

            yield (connected, scanned);
          }
        }(),
        builder: (context, snapshot) {
          final connected = snapshot.data?.$1 ?? [];
          final scanned = snapshot.data?.$2 ?? [];

          if (connected.isEmpty && scanned.isEmpty) {
            return widget.emptyWidget ?? const SizedBox();
          }
          return ListView.builder(
            itemCount: scanned.length + connected.length,
            itemBuilder: (context, index) {
              if (index < connected.length) {
                final device = connected[index];
                final childConnected = _getChildConnected(device, context);
                if (widget.enableTapConnected) {
                  InkWell(
                    onTap: () async => await _onTapConnected(module, device),
                    child: childConnected,
                  );
                }
                return childConnected;
              }
              final scannedIndex = index - connected.length;
              final scanResult = scanned[scannedIndex];

              final childScanned = _getChildScanned(scanResult, context);

              if (widget.enableTapScanned) {
                return InkWell(
                  onTap: () async => await _onTapScanned(module, scanResult),
                  child: childScanned,
                );
              }
              return childScanned;
            },
          );
        },
      ),
    );
  }

  Future<void> _onTapScanned(
      ModuleInterface module, ScanResult scanResult) async {
    final alt = (ScanResult scanResult) async {
      await scanResult.device.removeBond();
      return module.ble.connectDevice(scanResult.device.remoteId.str);
    };
    await (widget.onTapScanned ?? alt).call(scanResult);
    setState(() {});
  }

  Widget _getChildScanned(ScanResult scanResult, BuildContext context) {
    return widget.scannedBuilder?.call(scanResult) ??
        Card(
          child: SizedBox(
            height: 80,
            child: ListTile(
              title: Text('${scanResult.device.platformName}'),
              titleTextStyle: Theme.of(context)
                  .textTheme
                  .titleLarge
                  ?.copyWith(fontSize: 24),
              subtitle: Text(scanResult.device.remoteId.str),
              subtitleTextStyle: Theme.of(context)
                  .textTheme
                  .titleLarge
                  ?.copyWith(fontSize: 16),
              trailing: Text(
                'Scanned',
                style: Theme.of(context).textTheme.titleSmall,
              ),
            ),
          ),
        );
  }

  Future<void> _onTapConnected(
      ModuleInterface module, BluetoothDevice device) async {
    final alt = (device) => module.ble.disconnectDevice(device: device);
    await (widget.onTapConnected ?? alt).call(device);
    setState(() {});
  }

  Widget _getChildConnected(BluetoothDevice device, BuildContext context) {
    return widget.connectedBuilder?.call(device) ??
        Card(
          child: SizedBox(
            height: 80,
            child: ListTile(
              title: Text('${device.platformName}'),
              titleTextStyle: Theme.of(context)
                  .textTheme
                  .titleLarge
                  ?.copyWith(fontSize: 24),
              subtitle: Text(device.remoteId.str),
              subtitleTextStyle: Theme.of(context)
                  .textTheme
                  .titleLarge
                  ?.copyWith(fontSize: 16),
              trailing: Text(
                'Connected',
                style: Theme.of(context).textTheme.titleSmall,
              ),
            ),
          ),
        );
  }
}
